package jabadot;

import java.sql.*;
import java.io.*;
import java.util.*;

/** Create the database using raw JDBC; create the Admin user
 * using UserDBJDBC. Used on any new JabaDot installation;
 * contains strong echoes of The Initial Magick.
 */
public class CreateUserDatabase {

	/** The database. XXX get from JDConstants */
	protected final static String DB_URL = "jdbc:idb:userdb.prp";

	BufferedReader is;

	public static void main(String[] fn)
	throws ClassNotFoundException, SQLException, IOException {
		new CreateUserDatabase().create();
	}

	public CreateUserDatabase() throws IOException {
		is = new BufferedReader(new InputStreamReader(System.in));
	}

	/** Do the work */
	public void create()
	throws ClassNotFoundException, SQLException, IOException {

		// Load the database driver
		Class.forName("jdbc.idbDriver");

		System.out.println("Getting Connection");
		Connection conn = DriverManager.getConnection(
			DB_URL, "ian", "");	// user, password

		System.out.println("Creating Statement");
		Statement stmt = conn.createStatement();

		System.out.println("Creating table and index");
		stmt.executeUpdate("DROP TABLE userdb");
		stmt.executeUpdate("CREATE TABLE userdb (\n" +
			"name		char(12) PRIMARY KEY,\n" +
			"password	char(20),\n" +
			"fullName 	char(30),\n" +
			"email		char(60),\n" +
			"city		char(20),\n" +
			"prov		char(20),\n" +
			"country	char(20),\n" +
			"privs		int\n" +
			")");
		stmt.executeUpdate("CREATE INDEX nickIndex ON userdb (name)");
		stmt.close();
		conn.close();

		// Database is now empty. Create the first user, out of the void
		String nick = prompt("Enter admin name:");
		String pass = prompt("Enter password (will echo!):");
		String name = prompt("Enter full name:");
		String email = prompt("Enter admin's email:");
		String city = prompt("Enter this JabaDot's city:");
		String prov = prompt("Enter this JabaDot's province:");
		String country = prompt("Enter this JabaDot's country:");

		int privs = User.P_ADMIN | User.P_EDIT;
		User admin = new User(nick, pass, name, email,
			city, prov, country, privs);

		new UserDBJDBC().addUser(admin);

		return;			// All done with this program.
	}

	public String prompt(String message) throws IOException {
		String result = null;
		do {
			System.out.print(message); 
			System.out.print(' ');
			System.out.flush();
			result = is.readLine();
			if (result == null)	{
				System.err.println("Read end of file in prompt");
				System.exit(1);
			}
		} while (result.length() == 0);
		return result;
	}
}
