package jabadot;

import java.io.*;
import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * General Mail Servlet
 */
public class MailServlet extends HttpServlet {

	/** The mail host */
	protected String mailReadHost;
	/** The mail protocol, i.e., pop3 */
	protected String mailProtocol;
	/** Normally INBOX, same for everybody. */
	protected String mailReadRoot;
	/** The mail session, not the HttpSession */
	protected Session mailSession;

	/** The init method is called when the servlet is starting. */
	public void init(ServletConfig cfg) throws ServletException {

		mailProtocol = JDConstants.getProperty("jabadot.email.receive.protocol");
		mailReadHost = JDConstants.getProperty("jabadot.email.receive.host");
		mailReadRoot = JDConstants.getProperty("jabadot.email.receive.root");

		// Start with a JavaMail Session object, as usual
		mailSession = Session.getDefaultInstance(
			System.getProperties(), null);
		mailSession.setDebug(false);
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
	throws IOException {

		User user;

		// Session must exist; don't bother creating here if not.
		HttpSession httpSession = request.getSession(false);

		response.setContentType("text/html");
		PrintWriter out = response.getWriter();

		if (httpSession == null ||
		    (user = (User)(httpSession.getValue("jabadot.login"))) == null) {
			out.println("<H1>ERROR</H1><p>You must be logged in to use the Mail Service.");
			out.close();
			return;
		}

		// DEBUG
		out.println("Mail Setup: proto,host,root: " +
			mailProtocol + ", " + mailReadHost + ", " + mailReadRoot);

		try {
			// Find out from the HTTP request which function we're to do.
			String command = request.getParameter("command");

			// Do it.
			if (command == null || command.equals("list")) {
				list(out, user, mailReadRoot);
			}
			// XXX TODO
			//	compose/reply/replyAll
			//	delete
			//	move?
			else {
				out.println("Mail servlet - that function not written yet");
			}
		} catch (MessagingException mex) {
			out.println("<pre>");
			mex.printStackTrace(out);
		}
		out.close();
	}

	protected void list(PrintWriter out, User user, String root) throws MessagingException {

		boolean recursive = false;

		// Get a Store object for the given protocol
		Store store = mailSession.getStore(mailProtocol);

		// Connect to it.
		store.connect(mailReadHost, user.getName(), user.getPassword());

		// Get Folder object for root, and list it
		// If root name = "", getDefaultFolder(), else getFolder(root)
		Folder rf;
		if (root.length() != 0) {
			System.out.println("Getting folder " + root + ".");
			rf = store.getFolder(root);
		} else {
			System.out.println("Getting default folder.");
			rf = store.getDefaultFolder();
		}
		rf.open(Folder.READ_WRITE);

		if (rf.getType() == Folder.HOLDS_FOLDERS) {
			Folder[] f = rf.list();
			for (int i = 0; i < f.length; i++)
				listFolder(out, f[i], "", recursive);
		} else
				listFolder(out, rf, "", false);
		// XXX store.close(); ??
	}

	static void listFolder(PrintWriter out,
		Folder folder, String tab, boolean recurse)
	throws MessagingException {

		folder.open(Folder.READ_WRITE);
		out.println(tab + "Name: " + folder.getName() + '(' +
			folder.getFullName() + ')');
		if (!folder.isSubscribed())
			System.out.println(tab + "Not Subscribed");
		if ((folder.getType() & Folder.HOLDS_MESSAGES) != 0) {
			if (folder.hasNewMessages())
				System.out.println(tab + "Has New Messages");
			else
				System.out.println(tab + "No New Messages");
			Message[] msgs = folder.getMessages();
			for (int i=0; i<msgs.length; i++) {
				out.print("<li>");
				Message m = msgs[i];
				out.print(m.getFrom()[0]);
				out.print("--");
				out.print(m.getSubject());
				out.println();
			}
		}
		if ((folder.getType() & Folder.HOLDS_FOLDERS) != 0) {
			out.println(tab + "Is Directory");
		if (recurse) {
			Folder[] f = folder.list();
			for (int i=0; i < f.length; i++)
				listFolder(out, f[i], tab + "", recurse);
			}
		}
	}
}
